package com.isu.gaswellwatch.modbus.data.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.entity.Response;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/11/23 11:55
 */
@Slf4j
@Component
@SuppressWarnings("all")
public class Redis2DBPersistenceService {

    private static final String EXISTS_TABLE_SQL = "SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA" +
            " IN ('gaswellwatch', 'gas_well_watch') AND TABLE_NAME='$TableName$'";
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource(name = "stringRedisTemplate")
    private RedisTemplate redisTemplate;
    @Resource
    private Map<String, PersistenceHandler> persistenceHandlerMap;
    private RestTemplate restTemplate = new RestTemplate();

    @Scheduled(cron = "20/30 * * * * ? ")
    public void write() {
        Map<Long, String> idGatewayMappingMap = getOnlineGateway();
        HashOperations operations = this.redisTemplate.opsForHash();
        try (Cursor<String> cursor = this.redisTemplate.scan(ScanOptions.scanOptions()
                .match(PersistenceHandler.DEVICE_DATA_CACHE + "*").build())) {
            Long deviceId;
            String cacheKey, tableName;
            Map<String, Object> deviceMap;
            PersistenceHandler persistenceHandler;
            List<Map<String, Object>> existsTableList;
            while (cursor.hasNext()) {
                deviceMap = operations.entries(cacheKey = cursor.next());
                if (ObjectUtils.isEmpty(deviceMap)) {
                    continue;
                }
                deviceId = MapUtil.getLong(deviceMap, "deviceId");
                if (Objects.isNull(deviceId)) {
                    continue;
                }
                if (Objects.nonNull(idGatewayMappingMap)) {
                    operations.put(PersistenceHandler.DEVICE_DATA_CACHE + deviceId,
                            "online", String.valueOf(idGatewayMappingMap.containsKey(deviceId)));
                }
                persistenceHandler = persistenceHandlerMap.get("KNPCV1");
                tableName = "t_device_data_" + deviceId;
                existsTableList = this.jdbcTemplate.queryForList(StringUtils.replace(EXISTS_TABLE_SQL, "$TableName$", tableName));
                if (ObjectUtils.isEmpty(existsTableList)
                        || ObjectUtils.isEmpty(existsTableList.get(0))
                        || StringUtils.isBlank(MapUtil.getStr(existsTableList.get(0), "TABLE_NAME"))) {
                    persistenceHandler.createTable(tableName, deviceId);
                }
                persistenceHandler.insert(tableName, cacheKey);
            }
        }
    }

    private Map<Long, String> getOnlineGateway() {
        try {
            RequestEntity request = RequestEntity.get("http://127.0.0.1:9999/modbus-tcp/online").build();
            ResponseEntity<Response<List<String>>> response = this.restTemplate.exchange(request,
                    new ParameterizedTypeReference<Response<List<String>>>() {
                    });
            if (Objects.isNull(response)
                    || Objects.isNull(response.getBody())
                    || ObjectUtils.isEmpty(response.getBody().getData())) {
                return null;
            }
            List<Map<String, Object>> idGatewayMappingList = this.jdbcTemplate
                    .queryForList("select id, gateway_sn from devices where gateway_sn in (" +
                            response.getBody().getData().stream().map(value -> "'" + value + "'")
                                    .collect(Collectors.joining(",")) + ")");
            if (ObjectUtils.isEmpty(idGatewayMappingList)) {
                return null;
            }
            Map<Long, String> idGatewayMappingMap = new HashMap<Long, String>();
            for (Map<String, Object> map : idGatewayMappingList) {
                idGatewayMappingMap.put(MapUtil.getLong(map, "id"), MapUtil.getStr(map, "gateway_sn"));
            }
            return idGatewayMappingMap;
        } catch (Exception e) {
            log.error("Get online devices list fail", e);
            return null;
        }
    }

}
